@@ -42,6 +42,7 @@ import threading
 
 # Constants
 CL_CONTEXT_PLATFORM = 0x1084
+CL_PLATFORM_NAME = 0x0902
 CL_DEVICE_TYPE_CPU = 2
 CL_DEVICE_TYPE_GPU = 4
 CL_DEVICE_TYPE_ACCELERATOR = 8
@@ -50,11 +51,81 @@ CL_DEVICE_TYPE_ALL = 0xFFFFFFFF
 CL_DEVICE_TYPE = 0x1000
 CL_DEVICE_NAME = 0x102B
 CL_DEVICE_OPENCL_C_VERSION = 0x103D
-CL_PLATFORM_NAME = 0x0902
-CL_DEVICE_VENDOR = 0x102C
 CL_DEVICE_VENDOR_ID = 0x1001
-CL_DEVICE_GLOBAL_MEM_SIZE = 0x101F
+CL_DEVICE_MAX_COMPUTE_UNITS = 0x1002
+CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS = 0x1003
+CL_DEVICE_MAX_WORK_GROUP_SIZE = 0x1004
+CL_DEVICE_MAX_WORK_ITEM_SIZES = 0x1005
+CL_DEVICE_PREFERRED_VECTOR_WIDTH_CHAR = 0x1006
+CL_DEVICE_PREFERRED_VECTOR_WIDTH_SHORT = 0x1007
+CL_DEVICE_PREFERRED_VECTOR_WIDTH_INT = 0x1008
+CL_DEVICE_PREFERRED_VECTOR_WIDTH_LONG = 0x1009
+CL_DEVICE_PREFERRED_VECTOR_WIDTH_FLOAT = 0x100A
+CL_DEVICE_PREFERRED_VECTOR_WIDTH_DOUBLE = 0x100B
+CL_DEVICE_MAX_CLOCK_FREQUENCY = 0x100C
+CL_DEVICE_ADDRESS_BITS = 0x100D
+CL_DEVICE_MAX_READ_IMAGE_ARGS = 0x100E
+CL_DEVICE_MAX_WRITE_IMAGE_ARGS = 0x100F
+CL_DEVICE_MAX_MEM_ALLOC_SIZE = 0x1010
+CL_DEVICE_IMAGE2D_MAX_WIDTH = 0x1011
+CL_DEVICE_IMAGE2D_MAX_HEIGHT = 0x1012
+CL_DEVICE_IMAGE3D_MAX_WIDTH = 0x1013
+CL_DEVICE_IMAGE3D_MAX_HEIGHT = 0x1014
+CL_DEVICE_IMAGE3D_MAX_DEPTH = 0x1015
+CL_DEVICE_IMAGE_SUPPORT = 0x1016
+CL_DEVICE_MAX_PARAMETER_SIZE = 0x1017
+CL_DEVICE_MAX_SAMPLERS = 0x1018
 CL_DEVICE_MEM_BASE_ADDR_ALIGN = 0x1019
+CL_DEVICE_MIN_DATA_TYPE_ALIGN_SIZE = 0x101A
+CL_DEVICE_SINGLE_FP_CONFIG = 0x101B
+CL_DEVICE_GLOBAL_MEM_CACHE_TYPE = 0x101C
+CL_DEVICE_GLOBAL_MEM_CACHELINE_SIZE = 0x101D
+CL_DEVICE_GLOBAL_MEM_CACHE_SIZE = 0x101E
+CL_DEVICE_GLOBAL_MEM_SIZE = 0x101F
+CL_DEVICE_MAX_CONSTANT_BUFFER_SIZE = 0x1020
+CL_DEVICE_MAX_CONSTANT_ARGS = 0x1021
+CL_DEVICE_LOCAL_MEM_TYPE = 0x1022
+CL_DEVICE_LOCAL_MEM_SIZE = 0x1023
+CL_DEVICE_ERROR_CORRECTION_SUPPORT = 0x1024
+CL_DEVICE_PROFILING_TIMER_RESOLUTION = 0x1025
+CL_DEVICE_ENDIAN_LITTLE = 0x1026
+CL_DEVICE_AVAILABLE = 0x1027
+CL_DEVICE_COMPILER_AVAILABLE = 0x1028
+CL_DEVICE_EXECUTION_CAPABILITIES = 0x1029
+CL_DEVICE_QUEUE_PROPERTIES = 0x102A
+CL_DEVICE_NAME = 0x102B
+CL_DEVICE_VENDOR = 0x102C
+CL_DRIVER_VERSION = 0x102D
+CL_DEVICE_PROFILE = 0x102E
+CL_DEVICE_VERSION = 0x102F
+CL_DEVICE_EXTENSIONS = 0x1030
+CL_DEVICE_PLATFORM = 0x1031
+CL_DEVICE_DOUBLE_FP_CONFIG = 0x1032
+CL_DEVICE_HALF_FP_CONFIG = 0x1033
+CL_DEVICE_PREFERRED_VECTOR_WIDTH_HALF = 0x1034
+CL_DEVICE_HOST_UNIFIED_MEMORY = 0x1035
+CL_DEVICE_NATIVE_VECTOR_WIDTH_CHAR = 0x1036
+CL_DEVICE_NATIVE_VECTOR_WIDTH_SHORT = 0x1037
+CL_DEVICE_NATIVE_VECTOR_WIDTH_INT = 0x1038
+CL_DEVICE_NATIVE_VECTOR_WIDTH_LONG = 0x1039
+CL_DEVICE_NATIVE_VECTOR_WIDTH_FLOAT = 0x103A
+CL_DEVICE_NATIVE_VECTOR_WIDTH_DOUBLE = 0x103B
+CL_DEVICE_NATIVE_VECTOR_WIDTH_HALF = 0x103C
+CL_DEVICE_OPENCL_C_VERSION = 0x103D
+CL_DEVICE_LINKER_AVAILABLE = 0x103E
+CL_DEVICE_BUILT_IN_KERNELS = 0x103F
+CL_DEVICE_IMAGE_MAX_BUFFER_SIZE = 0x1040
+CL_DEVICE_IMAGE_MAX_ARRAY_SIZE = 0x1041
+CL_DEVICE_PARENT_DEVICE = 0x1042
+CL_DEVICE_PARTITION_MAX_SUB_DEVICES = 0x1043
+CL_DEVICE_PARTITION_PROPERTIES = 0x1044
+CL_DEVICE_PARTITION_AFFINITY_DOMAIN = 0x1045
+CL_DEVICE_PARTITION_TYPE = 0x1046
+CL_DEVICE_REFERENCE_COUNT = 0x1047
+CL_DEVICE_PREFERRED_INTEROP_USER_SYNC = 0x1048
+CL_DEVICE_PRINTF_BUFFER_SIZE = 0x1049
+CL_DEVICE_IMAGE_PITCH_ALIGNMENT = 0x104A
+CL_DEVICE_IMAGE_BASE_ADDRESS_ALIGNMENT = 0x104B
 CL_QUEUE_OUT_OF_ORDER_EXEC_MODE_ENABLE = 1
 CL_QUEUE_PROFILING_ENABLE = 2
 CL_PROGRAM_BUILD_LOG = 0x1183
@@ -71,6 +142,21 @@ CL_PROFILING_COMMAND_QUEUED = 0x1280
 CL_PROFILING_COMMAND_SUBMIT = 0x1281
 CL_PROFILING_COMMAND_START = 0x1282
 CL_PROFILING_COMMAND_END = 0x1283
+CL_PROGRAM_REFERENCE_COUNT = 0x1160
+CL_PROGRAM_CONTEXT = 0x1161
+CL_PROGRAM_NUM_DEVICES = 0x1162
+CL_PROGRAM_DEVICES = 0x1163
+CL_PROGRAM_SOURCE = 0x1164
+CL_PROGRAM_BINARY_SIZES = 0x1165
+CL_PROGRAM_BINARIES = 0x1166
+CL_PROGRAM_NUM_KERNELS = 0x1167
+CL_PROGRAM_KERNEL_NAMES = 0x1168
+CL_KERNEL_FUNCTION_NAME = 0x1190
+CL_KERNEL_NUM_ARGS = 0x1191
+CL_KERNEL_REFERENCE_COUNT = 0x1192
+CL_KERNEL_CONTEXT = 0x1193
+CL_KERNEL_PROGRAM = 0x1194
+CL_KERNEL_ATTRIBUTES = 0x1195
 
 
 #: ffi parser
